@@ -436,7 +436,7 @@ file_is_hidden (const gchar *name)
 }
 
 
-/* like g_basename but does not warn about NULL and does not
+/* like g_path_get_basename but does not warn about NULL and does not
  * alloc a new string. */
 G_CONST_RETURN gchar *
 file_name_from_path (const gchar *file_name)
@@ -582,7 +582,7 @@ make_directory_tree (GFile    *dir,
 	gboolean  success = TRUE;
 	GFile    *parent;
 
-	if (dir == NULL)
+	if ((dir == NULL) || g_file_query_exists (dir, NULL))
 		return TRUE;
 
 	parent = g_file_get_parent (dir);
@@ -703,12 +703,12 @@ gboolean
 is_mime_type (const char *mime_type,
 	      const char *pattern)
 {
-	return (strncasecmp (mime_type, pattern, strlen (pattern)) == 0);
+	return (strcasecmp (mime_type, pattern) == 0);
 }
 
 
 const char*
-get_file_mime_type (const char *filename,
+get_file_mime_type (const char *uri,
                     gboolean    fast_file_type)
 {
 	GFile      *file;
@@ -716,14 +716,14 @@ get_file_mime_type (const char *filename,
 	GError     *err = NULL;
  	const char *result = NULL;
 
- 	file = g_file_new_for_uri (filename);
+ 	file = g_file_new_for_uri (uri);
 	info = g_file_query_info (file,
 				  fast_file_type ?
 				  G_FILE_ATTRIBUTE_STANDARD_FAST_CONTENT_TYPE :
 				  G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE,
 				  0, NULL, &err);
 	if (info == NULL) {
-		g_warning ("could not get content type for %s: %s", filename, err->message);
+		g_warning ("could not get content type for %s: %s", uri, err->message);
 		g_clear_error (&err);
 	}
 	else {
@@ -776,21 +776,21 @@ path_list_dup (GList *path_list)
 
 
 guint64
-get_dest_free_space (const char *uri)
+get_dest_free_space (const char *path)
 {
 	guint64    freespace = 0;
 	GFile     *file;
 	GFileInfo *info;
 	GError    *err = NULL;
 
-	file = g_file_new_for_uri (uri);
+	file = g_file_new_for_path (path);
 	info = g_file_query_filesystem_info (file, G_FILE_ATTRIBUTE_FILESYSTEM_FREE, NULL, &err);
 	if (info != NULL) {
 		freespace = g_file_info_get_attribute_uint64 (info, G_FILE_ATTRIBUTE_FILESYSTEM_FREE);
 		g_object_unref (info);
 	}
 	else {
-		g_warning ("Could not get filesystem free space on volume that contains %s: %s", uri, err->message);
+		g_warning ("Could not get filesystem free space on volume that contains %s: %s", path, err->message);
 		g_error_free (err);
 	}
 	g_object_unref (file);
@@ -886,7 +886,7 @@ remove_local_directory (const char *path)
 }
 
 
-static const char *try_folder[] = { "~", "tmp", NULL };
+static const char *try_folder[] = { "cache", "~", "tmp", NULL };
 
 
 static char *
@@ -895,12 +895,14 @@ ith_temp_folder_to_try (int n)
 	const char *folder;
 
 	folder = try_folder[n];
-	if (strcmp (folder, "~") == 0)
+	if (strcmp (folder, "cache") == 0)
+		folder = g_get_user_cache_dir ();
+	else if (strcmp (folder, "~") == 0)
 		folder = g_get_home_dir ();
 	else if (strcmp (folder, "tmp") == 0)
 		folder = g_get_tmp_dir ();
 
-	return g_filename_to_uri (folder, NULL, NULL);
+	return g_strdup (folder);
 }
 
 
@@ -930,10 +932,10 @@ get_temp_work_dir (void)
 			g_free (folder);
 	}
 
-	if ((best_folder == NULL) || (strncmp (best_folder, "file://", 7) != 0))
+	if (best_folder == NULL)
 		return NULL;
-
-	template = g_strconcat (best_folder + strlen ("file://"), "/.fr-XXXXXX", NULL);
+	
+	template = g_strconcat (best_folder, "/.fr-XXXXXX", NULL);
 	result = mkdtemp (template);
 
 	if ((result == NULL) || (*result == '\0')) {
@@ -1106,6 +1108,7 @@ check_permissions (const char *uri,
 	GFile     *file;
 	GFileInfo *info;
 	GError    *err = NULL;
+	gboolean   default_permission_when_unknown = TRUE;
 
 	file = g_file_new_for_uri (uri);
 	info = g_file_query_info (file, "access::*", 0, NULL, &err);
@@ -1114,13 +1117,29 @@ check_permissions (const char *uri,
 		g_clear_error (&err);
 		result = FALSE;
 	}
-	else if ((mode & R_OK) && ! g_file_info_get_attribute_boolean (info, G_FILE_ATTRIBUTE_ACCESS_CAN_READ))
-		result = FALSE;
-	else if ((mode & W_OK) && ! g_file_info_get_attribute_boolean (info, G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE))
-		result = FALSE;
-	else if ((mode & X_OK) && ! g_file_info_get_attribute_boolean (info, G_FILE_ATTRIBUTE_ACCESS_CAN_EXECUTE))
-		result = FALSE;
-
+	else {
+		if ((mode & R_OK) == R_OK) {
+			if (g_file_info_has_attribute (info, G_FILE_ATTRIBUTE_ACCESS_CAN_READ))
+				result = (result && g_file_info_get_attribute_boolean (info, G_FILE_ATTRIBUTE_ACCESS_CAN_READ));
+			else
+				result = (result && default_permission_when_unknown);
+		}
+		
+		if ((mode & W_OK) == W_OK) {
+			if (g_file_info_has_attribute (info, G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE))
+				result = (result && g_file_info_get_attribute_boolean (info, G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE));
+			else
+				result = (result && default_permission_when_unknown);
+		}
+		
+		if ((mode & X_OK) == X_OK) {
+			if (g_file_info_has_attribute (info, G_FILE_ATTRIBUTE_ACCESS_CAN_EXECUTE))
+				result = (result && g_file_info_get_attribute_boolean (info, G_FILE_ATTRIBUTE_ACCESS_CAN_EXECUTE));
+			else
+				result = (result && default_permission_when_unknown);
+		}
+	}		
+	
 	g_object_unref (info);
 	g_object_unref (file);
 
